<?php defined( 'ABSPATH' ) || exit;

class The_Gift_Wrapper_Wrapping {

	/**
	 * If add_giftwrap_to_order() has already run on wp() hook
	 *
	 * @var boolean
	 */
	private $wrapped = false;

	/**
	 * Is there wrap in cart?
	 *
	 * @var boolean
	 */
	public $giftwrap_in_cart = false;

	/**
	 * Is cart full of only virtual product?
	 *
	 * @var boolean
	 */
	public $cart_virtual_products_only = false;

	/**
	 * Constructor
	 */
	 public function __construct() {

		// Maybe hide add to cart button after ajax remove_from_cart call
		add_action( 'wp_ajax_wcgwp_remove_from_cart',               array( $this, 'remove_from_cart' ) );
		add_action( 'wp_ajax_nopriv_wcgwp_remove_from_cart',        array( $this, 'remove_from_cart' ) );

		add_action( 'wp',                                           array( $this, 'wp' ) );

		add_action( 'template_redirect',                            array( $this, 'template_redirect' ) );

		// Load session data into Woo array
		add_filter( 'woocommerce_get_cart_item_from_session',       array( $this, 'get_cart_item_from_session' ), 20, 2 );

		// Just a quick check to see if wrap is in cart
		add_action( 'woocommerce_cart_loaded_from_session',         array( $this, 'check_cart_for_wrap' ), 10, 1 );

		// If thumbnail links aren't desired, remove them from cart as well
		add_filter( 'woocommerce_cart_item_permalink',              array( $this, 'remove_link_in_cart' ), 10, 3 );

		// maybe disable COD if gift wrap is in cart
		add_filter( 'woocommerce_available_payment_gateways',       array( $this, 'available_payment_gateways' ), 10, 1);

		// Unlink giftwrap item in order if desired
		add_filter( 'woocommerce_order_item_permalink',             array( $this, 'remove_link_in_order' ), 10, 3 );

		// Add more item data to the item data array
		add_filter( 'woocommerce_get_item_data',                    array( $this, 'get_item_data'), 10, 2 );

		// add class to table row in cart when it's giftwrap
		add_filter( 'woocommerce_cart_item_class',                  array( $this, 'cart_item_class' ), 11, 3 );

		// Filter the item meta display key, such as on order confirmation page
		add_filter( 'woocommerce_order_item_display_meta_key',      array( $this, 'order_item_display_meta_key' ), 10, 3 );

		// Add line items to order - adjust item before saving to order
		// Fires inside class-wc-checkout.php line 422
		add_action( 'woocommerce_checkout_create_order_line_item',  array( $this, 'checkout_create_order_line_item' ), 10, 3 );

	}

	/**
	 * Hook into Woocommerce_remove_from_cart AJAX and
	 * maybe hide gift wrap prompt when appropriate
	 *
	 * @return void
	 */
	public function remove_from_cart() {

		if ( defined('DOING_AJAX') && DOING_AJAX ) {
			wp_send_json_success(
				array(
					'hide' => ( 0 === WC()->cart->get_cart_contents_count() || $this->cart_excluded_from_wrap() ) ? true : false,
				)
			);
		}


	}

	/**
	 * Check if the cart is excluded from wrap offering
	 *
	 * @return bool
	 */
	public function cart_excluded_from_wrap() {

		$wrap_cat_id = get_option( 'wcgwp_category_id' );
		// WPML
		$wrap_cat_id = (int) apply_filters( 'wpml_object_id', $wrap_cat_id, 'product_cat', true );
		if ( empty( $wrap_cat_id )) {
			return true;
		}

		$cats = array();

		// Get category IDs of every item in cart (including gift wrap items)
		foreach( WC()->cart->get_cart() as $cart_item ) {
			$term_ids = wp_get_post_terms( $cart_item['product_id'], 'product_cat', array( 'fields' => 'ids' ) );
			// If it's in giftwrap category, remove its other categories
			if ( has_term( $wrap_cat_id, 'product_cat', $cart_item['product_id'] ) ) {
				$cats[] = array( $wrap_cat_id );
			} else {
				$cats[] = $term_ids;
			}
		}
		// Convert to one-dimensional array and remove duplicates
		$cats = array_unique( array_reduce( $cats, 'array_merge', array() ) );

		// The whole cart is excluded!
		if ( empty( array_diff( $cats, array( $wrap_cat_id ) ) ) ) {
			return true;
		}
		return false;

	}

	/**
	 * hooked into `wp` action
	 *
	 * @return void
	 */
	 public function wp() {

		// Leave if admin, not the right WC endpoint, or Gutenberg
		if ( is_admin() || ( ! is_cart() && ! is_checkout() ) || defined( 'REST_REQUEST' ) && REST_REQUEST ) {
			return;
		}

		if ( apply_filters( 'giftwrap_exclude_virtual_products', false ) ) {
			$this->cart_virtual_products_only = true;
		}

		if ( $this->count_wrap_products() < 1
			|| ( $this->cart_virtual_products_only && $this->cart_virtual_products_only() === true )
		) {
			return;
		}

		$giftwrap_display = wc_clean( get_option( 'wcgwp_display', array() ) );

		if ( ! is_array( $giftwrap_display ) ) {
			$giftwrap_display = str_split( $giftwrap_display, 17 );
		}

		if ( in_array( "before_cart", $giftwrap_display ) ) {
			add_action( 'woocommerce_before_cart', array( $this, 'before_cart' ) );
		}
		if ( in_array( "after_coupon", $giftwrap_display ) ) {
			add_action( 'woocommerce_before_cart_collaterals', array( $this, 'before_cart_collaterals' ) );
		}
		if ( in_array( "after_cart", $giftwrap_display ) ) {
			add_action( 'woocommerce_after_cart', array( $this, 'after_cart' ) );
		}
		if ( in_array( "before_checkout", $giftwrap_display ) ) {
			add_action( 'woocommerce_before_checkout_form', array( $this, 'before_checkout_form' ) );
		}
		if ( in_array( "after_checkout", $giftwrap_display ) ) {
			add_action( 'woocommerce_after_checkout_form', array( $this, 'after_checkout_form' ) );
		}

	}

	public function template_redirect() {

		if ( ! is_checkout() && ! is_cart() ) {
			return;
		}

		if ( ! apply_filters( 'wcgwp_add_giftwrap_to_order', true ) ) {
			return;
		}

		// Check $this->wrapped to avoid loop
		if ( ! $this->wrapped ) {
			$this->add_giftwrap_to_order();
			$this->wrapped = true;
		}

	}

	/**
	 * Check Gift Wrapper form nonce before continuing
	 *
	 * @return bool
	 */
	private function check_nonce() {

		// Check nonce
		if ( isset( $_POST['wcgwp_order_wrap_nonce'] ) && wp_verify_nonce( $_POST['wcgwp_order_wrap_nonce'], 'wcgwp_order_wrap' ) ) {
			return true;
		}
		error_log( 'Your Gift Wrapper cart/checkout template override is missing a nonce. Please update your template(s).' );
		return false;

	}

	/**
	 * Redirect to cart or checkout page as appropriate
	 *
	 * @return void
	 */
	private function redirect() {

		// POST/REDIRECT/GET to prevent wrap from showing back up after delete + refresh
		if ( isset( $_POST['wcgwp_submit_before_cart'] ) || isset( $_POST['wcgwp_submit_coupon'] ) || isset( $_POST['wcgwp_submit_after_cart'] ) ) {
			$this->wrapped = true;
			wp_safe_redirect( wc_get_cart_url(), 303 );
			exit; // not die() because inside hook
		}

		if ( isset( $_POST['wcgwp_submit_checkout'] ) || isset( $_POST['wcgwp_submit_after_checkout'] ) ) {
			$this->wrapped = true;
			wp_safe_redirect( wc_get_checkout_url(), 303 );
			exit;
		}
		// Could also maybe do redirect to referrer, just sayin

	}

	/**
	 * Use WC add_to_cart method to add cart/checkout wrap to order
	 *
	 * @param  int       $product_id
	 * @param  array     $notes
	 * @return void
	 * @throws Exception
	 */
	public function add_giftwrap( $product_id, $notes = array() ) {

		// Allow more than one gift wrap to cart
		$allow_multiple = get_option( 'wcgwp_number', 'no' );

		if ( $this->giftwrap_in_cart ) {

			$product = wc_get_product( $product_id );

			// For products sold individually
			if ( $product && $product->is_sold_individually() ) {
				$cart = WC()->cart->get_cart_contents();
				foreach ( $cart as $item ) {
					// Item is already in cart:
					if ( isset( $item['product_id'] ) && $item['product_id'] === $product_id ) {
						// @todo
						// $message = sprintf( __( 'You cannot add another "%s" to your cart.', 'woocommerce' ), $product->get_name() );
						/**
						 * Filters message about more than 1 product being added to cart.
						 *
						 * @param string $message Message.
						 * @param WC_Product $product_data Product data.
						 *
						 * @since 4.5.0
						 */
						// $message = apply_filters( 'woocommerce_cart_product_cannot_add_another_message', $message, $product );
						// wc_add_notice( $message, apply_filters( 'woocommerce_cart_item_removed_notice_type', 'warning' ) );
						$this->redirect();
					}
				}
			}

			if ( 'no' === $allow_multiple ) {

				$wrap_cat_id = get_option( 'wcgwp_category_id' );
				// WPML
				$wrap_cat_id = (int) apply_filters( 'wpml_object_id', $wrap_cat_id, 'product_cat', true );
				if ( empty( $wrap_cat_id ) ) {
					return;
				}

				// Crude but if wrap in cart and shouldn't overlap, remove existing.
				foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
					$_product_id = $cart_item['product_id'];
					$it_matches  = false;
					$terms       = get_the_terms( $_product_id, 'product_cat' );
					if ( $terms ) {
						foreach ( $terms as $term ) {
							if ( $term->term_id === $wrap_cat_id ) {
								$it_matches = true;
								break;
							}
						}
						if ( $it_matches ) {
							WC()->cart->remove_cart_item( $cart_item_key );
						}
					}
				}
			}
		}

		WC()->cart->add_to_cart( $product_id, 1, 0, array(), $notes );

	}

	/**
	 * Add gift wrapping to cart
	 *
	 * @return void
	 * @throws Exception
	 */
	 public function add_giftwrap_to_order() {

		if ( isset( $_POST['wcgwp_submit_before_cart'] ) ) {
			$nonce = $this->check_nonce();
			if ( ! $nonce ) {
				return;
			}
			$product_id = isset( $_POST['wcgwp_product_before_cart'] ) ? (int) $_POST['wcgwp_product_before_cart'] : false;
			if ( ! $product_id ) {
				return;
			}
			if ( isset( $_POST['wcgwp_note_before_cart'] ) && '' !== $_POST['wcgwp_note_before_cart'] ) {
				$notes = array( 'wcgwp_cart_note' => wc_sanitize_textarea( stripslashes( $_POST['wcgwp_note_before_cart'] ) ) );
			} else {
				$notes = null;
			}
			$this->add_giftwrap( $product_id, $notes );
		}

		if ( isset( $_POST['wcgwp_submit_coupon'] ) ) {
			$nonce = $this->check_nonce();
			if ( ! $nonce ) {
				return;
			}
			$product_id = isset( $_POST['wcgwp_product_coupon'] ) ? (int) $_POST['wcgwp_product_coupon'] : false;
			if ( ! $product_id ) {
				return;
			}
			$notes = $_POST['wcgwp_note_coupon'] != '' ? array( 'wcgwp_cart_note' => wc_sanitize_textarea( stripslashes( $_POST['wcgwp_note_coupon'] ) ) ) : null;
			$this->add_giftwrap( $product_id, $notes );
		}

		if ( isset( $_POST['wcgwp_submit_after_cart'] ) ) {
			$nonce = $this->check_nonce();
			if ( ! $nonce ) {
				return;
			}
			$product_id = isset( $_POST['wcgwp_product_after_cart'] ) ? (int) $_POST['wcgwp_product_after_cart'] : false;
			if ( ! $product_id ) {
				return;
			}
			$notes = $_POST['wcgwp_note_after_cart'] != '' ? array( 'wcgwp_cart_note' => wc_sanitize_textarea( stripslashes( $_POST['wcgwp_note_after_cart'] ) ) ) : null;
			$this->add_giftwrap( $product_id, $notes );
		}

		if ( isset( $_POST['wcgwp_submit_checkout'] ) ) {
			$nonce = $this->check_nonce();
			if ( ! $nonce ) {
				return;
			}
			$product_id = isset( $_POST['wcgwp_product_checkout'] ) ? (int) wc_clean( $_POST['wcgwp_product_checkout'] ) : false;
			if ( ! $product_id ) {
				return;
			}
			$notes = $_POST['wcgwp_note_checkout'] != '' ? array( 'wcgwp_cart_note' => wc_sanitize_textarea( stripslashes( $_POST['wcgwp_note_checkout'] ) ) ) : null;
			$this->add_giftwrap( $product_id, $notes );
		}

		if ( isset( $_POST['wcgwp_submit_after_checkout'] ) ) {
			$nonce = $this->check_nonce();
			if ( ! $nonce ) {
				return;
			}
			$product_id = isset( $_POST['wcgwp_product_after_checkout'] ) ? (int) $_POST['wcgwp_product_after_checkout'] : false;
			if ( ! $product_id ) {
				return;
			}
			$notes = $_POST['wcgwp_note_after_checkout'] != '' ? array( 'wcgwp_cart_note' => wc_sanitize_textarea( stripslashes( $_POST['wcgwp_note_after_checkout'] ) ) ) : false;
			$this->add_giftwrap( $product_id, $notes );
		}

		$this->redirect();

	}

	/**
	 * Is product gift wrap?
	 *
	 * @param  int|object $product (might be cart item)
	 * @return bool
	 */
	public function is_wrap( $product_id ) {

		$wrap_cat_id = get_option( 'wcgwp_category_id' );
		// WPML
		$wrap_cat_id = (int) apply_filters( 'wpml_object_id', $wrap_cat_id, 'product_cat', true );
		if ( empty( $wrap_cat_id ) ) {
			return false;
		}

		// Maybe get integer from object
		if ( is_object( $product_id ) && is_a( $product_id, 'WC_Order_Item_Product' ) ) {
			$product_id = $product_id->get_product_id();
		} else if ( isset( $product_id['data'] ) ) {
			$product_id = (int) $product_id['data']->get_id();
		} else {
			$product_id = absint( $product_id );
		}
		// We tried.
		if ( ! $product_id || ! is_int( $product_id ) ) {
			return false;
		}
		$terms = get_the_terms( $product_id, 'product_cat' );
		if ( $terms ) {
			foreach ( $terms as $term ) {
				if ( $term->term_id === $wrap_cat_id ) {
					return true;
				}
			}
		}
		return false;

	}

	/**
	 * Unlink giftwrap item in cart if desired
	 *
	 * @param  string $link          Cart item link, whether URL or blank
	 * @param  object $cart_item     Cart item
	 * @param  string $cart_item_key Cart item key
	 * @return bool
	 */
	 public function remove_link_in_cart( $link, $cart_item, $cart_item_key ) {

		$link = apply_filters( 'wcgwp_filter_link_in_cart', $link, $cart_item );
		if ( 'yes' === get_option( 'wcgwp_link', 'yes' ) ) {
			return $link;
		}
		if ( $this->is_wrap( $cart_item ) ) {
			$link = '';
		}
		return $link;

	}

	/**
	 * Set giftwrap_in_cart variable
	 * Loads with `woocommerce_cart_loaded_from_session` action hook
	 *
	 * @param  object $cart
	 * @return void
	 */
	 public function check_cart_for_wrap( $cart ) {

		$this->giftwrap_in_cart = false;
		if ( $this->wrap_in_cart( $cart ) ) {
			$this->giftwrap_in_cart = true;
		}

	}

	/**
	 * Check WooCommerce cart for wrap
	 *
	 * @param  object  $cart
	 * @return boolean
	 */
	public function wrap_in_cart( $cart ) {

		$wrap_cat_id = get_option( 'wcgwp_category_id' );
		// WPML filter
		$wrap_cat_id = (int) apply_filters( 'wpml_object_id', $wrap_cat_id, 'product_cat', true );
		if ( empty( $wrap_cat_id ) ) {
			return false;
		}

		if ( isset( $cart->cart_contents ) ) {
			foreach ( $cart->cart_contents as $item ) {
				$product_id = $item['product_id'];
				$terms = get_the_terms( $product_id, 'product_cat' );
				if ( $terms ) {
					foreach ( $terms as $term ) {
						if ( $term->term_id === $wrap_cat_id ) {
							return true;
						}
					}
				}
			}
		}
		return false;

	}

	/**
	 * Assign cart item values from session
	 *
	 * @param  array $cart_item Cart item data.
	 * @param  array $values	Cart item values.
	 * @return array
	 */
	 public function get_cart_item_from_session( $cart_item, $values ) {

		// Cart/checkout hooked general gift wrapping
		if ( isset( $values['wcgwp_cart_note'] ) ) {
			$cart_item['wcgwp_cart_note'] = $values['wcgwp_cart_note'];
		}
		return $cart_item;

	}

	/**
	 * Return array of products in gift wrap category
	 *
	 * @return null|array
	 */
	public function get_products() {

		$wrap_cat_id = get_option( 'wcgwp_category_id' );
		// WPML
		$wrap_cat_id = (int) apply_filters( 'wpml_object_id', $wrap_cat_id, 'product_cat', true );
		// Admin doesn't have a gift wrap category set!
		if ( empty( $wrap_cat_id ) ) {
			return array();
		}
		$suppress_filters = true;
		if ( class_exists( 'SitePress' ) ) { // for WPML
			$suppress_filters = false;
		}
		$category = get_term( $wrap_cat_id, 'product_cat' );
		if ( ! $category || is_wp_error( $category ) ) {
			return array();
		}
		// Returns objects by default
		$args = apply_filters( 'wcgwp_post_args', array(
			'category'			=> $category->slug,
			'status'			=> 'publish',
			'stock_status'		=> 'instock',
			'limit'				=> '-1',
			'orderby'			=> apply_filters( 'wcgwp_orderby', 'date' ),
			'order'				=> apply_filters( 'wcgwp_order', 'DESC' ),
			'suppress_filters'	=> $suppress_filters, // accord for WPML using filter (explained in link below)
		), $category );

		// https://github.com/woocommerce/woocommerce/wiki/wc_get_products-and-WC_Product_Query
		return apply_filters( 'wcgwp_wrap_posts', wc_get_products( $args ) );

	}

	/**
	 * Count array of products in gift wrap category
	 *
	 * @return int
	 */
	public function count_wrap_products() {

		return count( $this->get_products() );

	}

	/**
	 * Wrapper function for gift_wrap_action()
	 *
	 * @return void
	 */
	public function before_cart() {

		$this->gift_wrap_action( '_before_cart' );

	}

	/**
	 * Wrapper function for gift_wrap_action()
	 *
	 * @return void
	 */
	public function before_cart_collaterals() {

		$this->gift_wrap_action( '_coupon' );

	}

	/**
	 * Wrapper function for gift_wrap_action()
	 *
	 * @return void
	 */
	public function after_cart() {

		$this->gift_wrap_action( '_after_cart' );

	}

	/**
	 * Wrapper function for gift_wrap_action()
	 *
	 * @return void
	 */
	public function before_checkout_form() {

		$this->gift_wrap_action( '_checkout' );

	}

	/**
	 * Wrapper function for gift_wrap_action()
	 *
	 * @return void
	 */
	public function after_checkout_form() {

		$this->gift_wrap_action( '_after_checkout' );

	}

	/**
	 * Add gift wrap options to cart/checkout action hooks
	 *
	 * @param string $label
	 * @return void
	 */
	 public function gift_wrap_action( $label ) {

		$list = $this->get_products();

		if ( ! apply_filters( 'wcgwp_continue_gift_wrap_action', true, $list, $label ) ) return;

		$giftwrap_details = wc_sanitize_textarea( get_option( 'wcgwp_details' ) );
		?>

		<div class="wc-giftwrap giftwrap<?php echo esc_attr( $label ); ?> <?php echo esc_attr( $this->extra_class() ); ?>">

			<?php
			// If modal version
			if ( 'yes' === get_option( 'wcgwp_modal', 'no' ) ) {

				wc_get_template( 'wcgwp/modal.php', array( 'label' => $label, 'list' => $list, 'giftwrap_details' => $giftwrap_details, 'show_thumbs' => $this->show_thumbs() ), '', GIFTWRAPPER_PLUGIN_DIR . 'templates/');

			// Non-modal version
			} else {

			// New template since version 4.4, old was wcgwp/giftwrap-list-cart.php (deprecated)
			wc_get_template( 'wcgwp/giftwrap-list.php', array( 'label' => $label, 'list' => $list, 'giftwrap_details' => $giftwrap_details, 'show_thumbs' => $this->show_thumbs() ), '', GIFTWRAPPER_PLUGIN_DIR . 'templates/');

			} ?>
		</div>

	<?php

	}

	/**
	 * Check if the cart contains virtual product
	 * via Remi Corson, 10/2013
	 *
	 * @return bool
	 */
	 public function cart_virtual_products_only() {

		$all_virtual_products = false;
		$virtual_product_num = 0;

		$cart = WC()->cart->get_cart();

		if ( ! $cart ){
			return false;
		}

		foreach ( $cart as $cart_item ) {

			// Get product ID and '_virtual' post meta
			$product_id = $cart_item['product_id'];

			// Update $has_virtual_product if product is virtual
			if ( 'yes' === get_post_meta( $product_id, '_virtual', true ) ) {
				$virtual_product_num += 1;
			}
		}

		if ( count( $cart ) === $virtual_product_num ) {
			$all_virtual_products = true;
		}
		return apply_filters( 'wcgwp_virtual_products_only', $all_virtual_products );

	}

	/**
	 * Add conditional classes to giftwrap wrapper div
	 *
	 * @return string
	 */
	 public function extra_class() {

		$extra_class = '';
		if ( $this->giftwrap_in_cart === false || ( $this->giftwrap_in_cart === true && 'yes' === get_option( 'wcgwp_number', 'no' ) ) ) {
			 $extra_class = ' wcgwp_could_giftwrap';
		}
		return apply_filters( 'wcgwp_extra_wrapper_class', $extra_class );

	}

	/**
	 * Whether to show gift wrap product thumbnails...
	 *
	 * @return bool
	 */
	 public function show_thumbs() {

		if ( 'yes' === get_option( 'wcgwp_show_thumb', 'yes' ) ) {
			return true;
		}
		return false;

	}

	/**
	 * Maybe disable payment gateways if gift wrap in cart
	 *
	 * @param  array      $gateways
	 * @return null|array
	 */
	 public function available_payment_gateways( $gateways ) {

		if ( ! $this->giftwrap_in_cart ) {
			return $gateways;
		}
		if ( apply_filters( 'wcgwp_remove_cod_gateway', false ) ) {
			if ( isset( $gateways['cod'] ) ) {
				unset( $gateways['cod'] );
			}
		}
		return apply_filters( 'wcgwp_change_gateways', $gateways );

	}

	/**
	 * Unlink giftwrap item in order if desired
	 *
	 * @param  string $link  Order item link, whether URL or blank
	 * @param  object $item  Order item
	 * @param  object $order Order
	 * @return string
	 */
	 public function remove_link_in_order( $link, $item, $order ) {

		// Exit if we're not dealing with wrap
		if ( ! $this->is_wrap( $item ) ) {
			return $link;
		}
		if ( get_option( 'wcgwp_link', 'yes' ) == 'no' ) {
			$link = '';
		}
		return $link;

	}

	/**
	 * Display user's note on the cart itemization
	 *
	 * @param  array $item_data
	 * @param  array $cart_item
	 * @return array
	 */
	 public function get_item_data( $item_data, $cart_item ) {

		// Cart/checkout hooked general gift wrapping
		if ( ! isset( $cart_item['wcgwp_cart_note'] ) ) {
			return $item_data;
		}

		$item_data[] = array(
			'key'   => __( 'Note', 'woocommerce-gift-wrapper' ),
			'value' => $cart_item['wcgwp_cart_note'],
		);
		return $item_data;

	}

	/**
	 * Alter CSS class names of wrapping cart items <tr> in cart table
	 *
	 * @param  string  $class
	 * @param  array   $cart_item
	 * @param  string  $cart_item_key
	 * @return string
	 */
	public function cart_item_class( $class, $cart_item, $cart_item_key ) {

		if ( $this->is_wrap( $cart_item ) ) {
			$class .= ' wcgwp-wrap-product ';
		}
		return $class;

	}

	/**
	 * Filter the item meta display key, such as on order confirmation page
	 *
	 * @param  string $display_key    Display key
	 * @param  object $meta           WC_Meta_Data
	 * @param  object $order_item     WC_Order_Item_Product
	 * @return string
	 */
	 public function order_item_display_meta_key( $display_key, $meta, $order_item ) {

		if ( $display_key == 'wcgwp_note' ) {
			$display_key = str_replace( 'wcgwp_note', __( 'Note', 'woocommerce-gift-wrapper' ), $display_key );
		}
		return $display_key;

	}

	/**
	 * Include add-ons line item meta.
	 *
	 * @param  object $item           WC_Order_Item_Product
	 * @param  string $cart_item_key  Cart item key.
	 * @param  array  $values         Order item values.
	 * @return object
	 */
	 public function checkout_create_order_line_item( $item, $cart_item_key, $values ) {

		if ( isset( $values['wcgwp_cart_note'] ) ) {
			$item->add_meta_data( 'wcgwp_note', $values['wcgwp_cart_note'] );
		}
		return $item;

	}

}