<?php

defined( 'ABSPATH' ) || exit; // Exit if accessed directly

final class The_Gift_Wrapper {

	/**
	 * Single instance of the The_Gift_Wrapper class
	 *
	 * @var The_Gift_Wrapper
	 */
	protected static $_instance = null;

	/**
	 * Wrapping instance.
	 *
	 * @var The_Gift_Wrapper_Wrapping
	 */
	public $wrapping = null;

	/**
	 * Settings instance.
	 *
	 * @var The_Gift_Wrapper_Settings
	 */
	public $settings = null;


	/**
	 * Main Gift Wrapper Instance
	 *
	 * Ensures only one instance of Gift Wrapper is loaded or can be loaded
	 *
	 * @since 1.0
	 * @static
	 * @see WC_Gift_Wrap()
	 * @return The_Gift_Wrapper - Main instance
	 */
	public static function instance() {

		if ( ! isset( self::$_instance ) && ! ( self::$_instance instanceof The_Gift_Wrapper ) ) {
			self::$_instance = new The_Gift_Wrapper;
		}
		return self::$_instance;

	}

	/**
	 * Cloning instances of the class is forbidden
	 */
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; uh?', 'woocommerce-gift-wrapper' ), GIFTWRAPPER_VERSION );
	}

	/**
	 * Unserializing instances of the class is forbidden
	 */
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; uh?', 'woocommerce-gift-wrapper' ), GIFTWRAPPER_VERSION );
	}

	/**
	 * Constructor
	 */
	public function __construct() {

		register_activation_hook( GIFTWRAPPER_PLUGIN_FILE, array( $this, 'activation_hook' ) );

		$this->define_constants();
		$this->includes();
		$this->hooks();

	}

	/**
	 * Deactivate the free version of WaterWoo if active
	 *
	 * @return void
	 */
	public function activation_hook() {

		if ( ! function_exists( 'is_plugin_active' ) ) {
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		if ( is_plugin_active( 'woocommerce-gift-wrapper-plus/woocommerce-gift-wrapper-plus.php' ) ) {
			deactivate_plugins( 'woocommerce-gift-wrapper-plus/woocommerce-gift-wrapper-plus.php' );
		}

	}

	/**
	 * Define constants used by plugin
	 *
	 * @return void
	 */
	private function define_constants() {

		if ( ! defined( 'GIFTWRAPPER_PLUGIN_DIR' ) ) {
			define( 'GIFTWRAPPER_PLUGIN_DIR', plugin_dir_path( GIFTWRAPPER_PLUGIN_FILE ) );
		}
		if ( ! defined( 'GIFTWRAPPER_PLUGIN_BASE_FILE' ) ) {
			define( 'GIFTWRAPPER_PLUGIN_BASE_FILE', plugin_basename(GIFTWRAPPER_PLUGIN_FILE ) );
		}
		if ( ! defined( 'GIFTWRAPPER_VERSION' ) ) {
			define( 'GIFTWRAPPER_VERSION', '5.2.2' );
		}

	}

	/**
	 * Include required files
	 *
	 * @return void
	 */
	public function includes() {

		include_once GIFTWRAPPER_PLUGIN_DIR . 'includes/class-gift-wrapper-admin-notices.php';
		include_once GIFTWRAPPER_PLUGIN_DIR . 'includes/class-gift-wrapper-settings.php';
		include_once GIFTWRAPPER_PLUGIN_DIR . 'includes/class-gift-wrapper-settings-product.php';
		include_once GIFTWRAPPER_PLUGIN_DIR . 'includes/class-gift-wrapper-wrapping.php';

	}

	/**
	 * Hooked functions
	 *
	 * @return void
	 */
	public function hooks() {

		add_action( 'plugins_loaded',           array( $this, 'plugins_loaded' ) );

		add_action( 'admin_enqueue_scripts',    array( $this, 'admin_enqueue_scripts' ), 10, 1 );

		add_action( 'wp_enqueue_scripts',       array( $this, 'wp_enqueue_scripts' ), 51 );

		add_action( 'wp_login',                 array( $this, 'wp_login' ), 15, 2 );

	}

	/**
	 * Load the localization
	 *
	 * @return void
	 */
	public function plugins_loaded() {

		if ( function_exists( 'determine_locale' ) ) {
			$locale = determine_locale();
			$locale = apply_filters( 'plugin_locale', $locale, 'woocommerce-gift-wrapper' );
			unload_textdomain( 'woocommerce-gift-wrapper' );
			load_textdomain( 'woocommerce-gift-wrapper', WP_LANG_DIR . '/woocommerce-gift-wrapper/woocommerce-gift-wrapper-' . $locale . '.mo' );
			load_plugin_textdomain( 'woocommerce-gift-wrapper', false, dirname( plugin_basename( GIFTWRAPPER_PLUGIN_FILE ) ) . '/lang' );
		} else {
			load_plugin_textdomain( 'woocommerce-gift-wrapper', false, dirname( plugin_basename( GIFTWRAPPER_PLUGIN_FILE ) ) . '/lang' );
		}

		$this->update_db();

		if ( is_admin() ) {
			new The_Gift_Wrapper_Admin_Notices();
			$this->settings = new The_Gift_Wrapper_Settings();
			new The_Gift_Wrapper_Settings_Product();
		}

		$this->wrapping = new The_Gift_Wrapper_Wrapping();

	}

	/**
	 * Enqueue admin-side scripts
	 *
	 * @param  string $hook_suffix
	 * @return void
	 */
	public function admin_enqueue_scripts( $hook_suffix ) {

		wp_register_script( 'wcgiftwrap-admin-js', plugins_url( '/assets/js/gift-wrapper-admin.min.js', GIFTWRAPPER_PLUGIN_FILE ), array( 'jquery' ), GIFTWRAPPER_VERSION );
		wp_register_style( 'wcgiftwrap-admin-css', plugins_url( '/assets/css/gift-wrapper-admin.min.css', GIFTWRAPPER_PLUGIN_FILE ), array(), GIFTWRAPPER_VERSION );

		if ( 'woocommerce_page_wc-settings' === $hook_suffix && isset( $_GET['tab'] ) && 'gift-wrapper' === $_GET['tab'] ) {
			wp_enqueue_script( 'wcgiftwrap-admin-js' );
			wp_enqueue_style( 'wcgiftwrap-admin-css' );
			return;
		}

		$screen = get_current_screen();
		$screen_id = $screen ? $screen->id : '';
		if ( 'product'=== $screen_id || 'edit-product' === $screen_id ) {
			wp_enqueue_style( 'wcgiftwrap-admin-css' );
		}

	}

	/**
	 * Enqueue scripts
	 *
	 * @return void
	 */
	public function wp_enqueue_scripts() {

		if ( ! is_cart() && ! is_checkout() ) return;

		$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

		if ( 'yes' === get_option( 'wcgwp_modal' ) ) { // Modal

			wp_enqueue_style( 'wcgiftwrap-modal-css', plugins_url( '/assets/css/gift-wrapper-modal' . $suffix . '.css', GIFTWRAPPER_PLUGIN_FILE ), array(), GIFTWRAPPER_VERSION );

			wp_enqueue_script( 'wcgwp-modal', plugins_url( 'assets/js/modal-vanilla/modal.min.js', GIFTWRAPPER_PLUGIN_FILE ), array(), GIFTWRAPPER_VERSION, true );
			wp_enqueue_script( 'wcgwp-modal-cart-checkout', plugins_url( 'assets/js/gift-wrapper-modal' . $suffix . '.js', GIFTWRAPPER_PLUGIN_FILE ), array( 'wcgwp-modal' ), GIFTWRAPPER_VERSION, true );
			wp_localize_script( 'wcgwp-modal-cart-checkout', 'wcgwpModal', array(
					'ajaxurl' => admin_url( 'admin-ajax.php' ),
					'number' => sanitize_text_field( get_option( 'wcgwp_number', 'no' ) ),
					'wrapInCart' => WC_Gift_Wrap()->wrapping->giftwrap_in_cart,
					'replaceText' => esc_js( __( 'Are you sure you want to replace the gift wrap in your cart?', 'woocommerce-gift-wrapper' ) ),
				)
			);

			if ( in_array( 'after_coupon', get_option( 'wcgwp_display', array() ) ) ) {
				wp_add_inline_script(
					'wcgwp-modal-cart-checkout',
					'var wcgwpWidth=jQuery(".cart-collaterals").css("width"),wcgwpFloat=jQuery(".cart-collaterals").css("float");if("none"!==wcgwpFloat){wcgwpWidth&&wcgwpFloat&&jQuery(".wc-giftwrap.giftwrap_coupon").attr("style","width:"+wcgwpWidth+";float:"+wcgwpFloat+";clear:none")}',
					'before'
				);
			}

		} else { // slideToggle

			wp_enqueue_style( 'wcgiftwrap-css', plugins_url( '/assets/css/gift-wrapper-slide' . $suffix . '.css', GIFTWRAPPER_PLUGIN_FILE ), array(), null );

			wp_enqueue_script( 'wcgwp-slide-cart-checkout', plugins_url( 'assets/js/gift-wrapper-slide' . $suffix . '.js', GIFTWRAPPER_PLUGIN_FILE ), array( 'jquery' ), null, true );
			wp_localize_script( 'wcgwp-slide-cart-checkout', 'wcgwpSlide', array(
					'ajaxurl'       => admin_url( 'admin-ajax.php' ),
					'number'        => sanitize_text_field( get_option( 'wcgwp_number', 'no' ) ),
					'wrapInCart'    => WC_Gift_Wrap()->wrapping->giftwrap_in_cart,
					'replaceText'   => esc_js( __( 'Are you sure you want to replace the gift wrap in your cart?', 'woocommerce-gift-wrapper' ) ),
				)
			);

			if ( in_array( 'after_coupon', get_option( 'wcgwp_display', array() ) ) ) {
				wp_add_inline_script(
					'wcgwp-slide-cart-checkout',
					'var wcgwpWidth=jQuery(".cart-collaterals").css("width"),wcgwpFloat=jQuery(".cart-collaterals").css("float");if("none"!==wcgwpFloat){wcgwpWidth&&wcgwpFloat&&jQuery(".wc-giftwrap.giftwrap_coupon").attr("style","width:"+wcgwpWidth+";float:"+wcgwpFloat+";clear:none")}',
					'before'
				);
			}

		}

	}

	/**
	 * Fires after the user has successfully logged in
	 * - DB updates
	 *
	 * @param  string $user_login
	 * @param  object  WP_User $user
	 * @return void
	 */
	public function wp_login( $user_login, $user ) {

		$admin = false;
		foreach ( $user->roles as $role ) {
			if ( 'administrator' === $role ) {
				$admin = true;
				break;
			}
		}
		if ( ! $admin ) return;
		$this->update_db();

	}

	/**
	 * Update Woo order item meta in version 4.0.3
	 *
	 * @since  4.0.3
	 * @return void
	 */
	public function update_db() {

		if ( false === get_option( 'wcGIFTWRAPPER_VERSION' ) && get_option( 'giftwrap_category_id' ) ) {
			global $wpdb;
			$old = 'gift_wrap_note';
			$new = 'wcgwp_note';
			$wpdb->query( $wpdb->prepare(
				"UPDATE {$wpdb->prefix}woocommerce_order_itemmeta SET meta_key = REPLACE( meta_key, %s, %s )", $old, $new
			) );
		}

		$just_upgraded = false;
		$version = preg_replace( '/[^0-9.].*/', '', get_option( 'wcGIFTWRAPPER_VERSION', '0.1' ) );

		if ( version_compare( $version, '4.2.2', '<' ) ) {
			$just_upgraded = true; // there has just been an upgrade to >= 4.2.2!
		}

		if ( $just_upgraded ) {

			$options = array(
				'giftwrap_category_id'      => 'wcgwp_category_id',
				'giftwrap_details'          => 'wcgwp_details',
				'giftwrap_display'          => 'wcgwp_display',
				'giftwrap_link'             => 'wcgwp_link',
				'giftwrap_modal'            => 'wcgwp_modal',
				'giftwrap_number'           => 'wcgwp_number',
				'giftwrap_show_thumb'       => 'wcgwp_show_thumb',
				'giftwrap_textarea_limit'   => 'wcgwp_textarea_limit',
			);

			foreach( $options as $old => $new ) {
				if ( get_option( $old ) ) {
					update_option( $new, get_option( $old ) );
					delete_option( $old );
				}
			}

			// clean up unused options in the DB
			$disused = array(
				'giftwrap_button',
				'giftwrap_header',
				'giftwrap_text_label',
			);

			foreach( $disused as $option ) {
				delete_option( $option );
			}

			update_option( 'wcGIFTWRAPPER_VERSION', GIFTWRAPPER_VERSION, 'no' );

		}

	}

} // End class The_Gift_Wrapper